@@ -16,8 +16,10 @@ typedef struct {
 	gchar *bundle_version;
 	gchar *bundle_description;
 	gchar *bundle_build;
+	gchar *bundle_hash;
 	gchar *status;
 	RaucChecksum checksum;
+	gchar *installed_txn;
 	gchar *installed_timestamp;
 	guint32 installed_count;
 	gchar *activated_timestamp;
@@ -72,6 +74,13 @@ void r_slot_free(gpointer value);
 
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(RaucSlot, r_slot_free);
 
+/**
+ * Clears the data in a RaucSlotStatus.
+ *
+ * @param slotstatus a RaucSlotStatus
+ */
+void r_slot_clear_status(RaucSlotStatus *slotstatus);
+
 /**
  * Frees the memory allocated by the RaucSlotStatus.
  *
@@ -108,7 +117,7 @@ G_GNUC_WARN_UNUSED_RESULT;
  *
  * @return string representation of slot state
  */
-gchar* r_slot_slotstate_to_str(SlotState slotstate)
+const gchar* r_slot_slotstate_to_str(SlotState slotstate)
 G_GNUC_WARN_UNUSED_RESULT;
 
 /**
@@ -206,7 +215,7 @@ G_GNUC_WARN_UNUSED_RESULT;
  * @param slots GHashTable of system slots
  * @param class name of class to find all slots for
  *
- * @return list of pointers to all memers of slots hash table that are of
+ * @return list of pointers to all members of slots hash table that are of
  *         selected class.
  */
 GList* r_slot_get_all_of_class(GHashTable *slots, const gchar* class)
@@ -218,7 +227,7 @@ G_GNUC_WARN_UNUSED_RESULT;
  * @param slots GHashTable of system slots
  * @param parent Slot to find children for
  *
- * @return list of pointers to all memers of slots hash table that are children
+ * @return list of pointers to all members of slots hash table that are children
  *         of given slot.
  */
 GList* r_slot_get_all_children(GHashTable *slots, RaucSlot *parent)
