@@ -4,6 +4,8 @@
 #include "bootchooser.h"
 #include "config_file.h"
 #include "context.h"
+#include "event_log.h"
+#include "install.h"
 #include "manifest.h"
 #include "mount.h"
 #include "utils.h"
@@ -156,6 +158,182 @@ out:
 	return res;
 }
 
+#define RAUC_LOG_EVENT_CONF_PREFIX "log"
+
+static gboolean r_event_log_parse_config_sections(GKeyFile *key_file, RaucConfig *config, GError **error)
+{
+	gsize group_count;
+	g_auto(GStrv) groups = NULL;
+	gint tmp_maxfiles;
+
+	g_return_val_if_fail(key_file, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	g_assert_null(config->loggers);
+
+	/* parse [log.*] sections */
+	groups = g_key_file_get_groups(key_file, &group_count);
+	for (gchar **group = groups; *group != NULL; group++) {
+		GError *ierror = NULL;
+		g_autoptr(REventLogger) logger = NULL;
+		const gchar *logger_name;
+		g_autofree gchar *log_format = NULL;
+		gsize entries;
+
+		if (!g_str_has_prefix(*group, RAUC_LOG_EVENT_CONF_PREFIX "."))
+			continue;
+
+		logger_name = *group + strlen(RAUC_LOG_EVENT_CONF_PREFIX ".");
+
+		logger = g_new0(REventLogger, 1);
+		logger->name = g_strdup(logger_name);
+
+		/* 'filename' option is currently mandatory for a logger group */
+		logger->filename = key_file_consume_string(key_file, *group, "filename", &ierror);
+		if (!logger->filename) {
+			g_propagate_error(error, ierror);
+			return FALSE;
+		}
+		/* relative paths are resolved relative to the data-directory */
+		if (!g_path_is_absolute(logger->filename)) {
+			gchar *abspath;
+			if (!config->data_directory) {
+				g_set_error_literal(
+						error,
+						G_KEY_FILE_ERROR,
+						G_KEY_FILE_ERROR_INVALID_VALUE,
+						"Relative filename requires data-directory to be set");
+				return FALSE;
+			}
+
+			abspath = g_build_filename(config->data_directory, logger->filename, NULL);
+			g_free(logger->filename);
+			logger->filename = abspath;
+		}
+
+		log_format = key_file_consume_string(key_file, *group, "format", &ierror);
+		if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+			g_clear_pointer(&log_format, g_free);
+			log_format = g_strdup("readable");
+			g_clear_error(&ierror);
+		} else if (ierror) {
+			g_propagate_error(error, ierror);
+			return FALSE;
+		}
+
+		if (g_strcmp0(log_format, "readable") == 0) {
+			logger->format = R_EVENT_LOGFMT_READABLE;
+		} else if (g_strcmp0(log_format, "short") == 0) {
+			logger->format = R_EVENT_LOGFMT_READABLE_SHORT;
+		} else if (g_strcmp0(log_format, "json") == 0) {
+#if ENABLE_JSON
+			logger->format = R_EVENT_LOGFMT_JSON;
+#else
+			g_set_error(
+					error,
+					G_KEY_FILE_ERROR,
+					G_KEY_FILE_ERROR_INVALID_VALUE,
+					"Invalid log format %s. RAUC is compiled without JSON support", log_format);
+			return FALSE;
+#endif
+		} else if (g_strcmp0(log_format, "json-pretty") == 0) {
+#if ENABLE_JSON
+			logger->format = R_EVENT_LOGFMT_JSON_PRETTY;
+#else
+			g_set_error(
+					error,
+					G_KEY_FILE_ERROR,
+					G_KEY_FILE_ERROR_INVALID_VALUE,
+					"Invalid log format %s. RAUC is compiled without JSON support", log_format);
+			return FALSE;
+#endif
+		} else {
+			g_set_error(
+					error,
+					G_KEY_FILE_ERROR,
+					G_KEY_FILE_ERROR_INVALID_VALUE,
+					"Unknown log format '%s'", log_format);
+			return FALSE;
+		}
+
+		logger->events = g_key_file_get_string_list(key_file, *group, "events", &entries, &ierror);
+		if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+			logger->events = g_malloc(2 * sizeof(gchar *));
+			logger->events[0] = g_strdup("all");
+			logger->events[1] = NULL;
+			g_clear_error(&ierror);
+		} else if (ierror) {
+			g_propagate_error(error, ierror);
+			return FALSE;
+		}
+
+		if (g_strv_length(logger->events) > 1 && g_strv_contains((const gchar * const *) logger->events, "all")) {
+			g_set_error(
+					error,
+					G_KEY_FILE_ERROR,
+					G_KEY_FILE_ERROR_INVALID_VALUE,
+					"Event type 'all' cannot be combined");
+			return FALSE;
+		}
+		for (gsize j = 0; j < entries; j++) {
+			if (!r_event_log_is_supported_type(logger->events[j])) {
+				g_set_error(
+						error,
+						G_KEY_FILE_ERROR,
+						G_KEY_FILE_ERROR_INVALID_VALUE,
+						"Unsupported event log type '%s'", logger->events[j]);
+				return FALSE;
+			}
+		}
+		g_key_file_remove_key(key_file, *group, "events", NULL);
+
+		logger->maxsize = key_file_consume_binary_suffixed_string(key_file, *group, "max-size", &ierror);
+		if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+			g_clear_error(&ierror);
+		} else if (ierror) {
+			g_propagate_error(error, ierror);
+			return FALSE;
+		}
+
+		tmp_maxfiles = key_file_consume_integer(key_file, *group, "max-files", &ierror);
+		if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+			tmp_maxfiles = 10;
+			g_clear_error(&ierror);
+		} else if (ierror) {
+			g_propagate_error(error, ierror);
+			return FALSE;
+		}
+		if (tmp_maxfiles < 1) {
+			g_set_error_literal(
+					error,
+					G_KEY_FILE_ERROR,
+					G_KEY_FILE_ERROR_INVALID_VALUE,
+					"Value for 'max-files' must be >= 1");
+			return FALSE;
+		} else if (tmp_maxfiles > 1000) {
+			g_set_error_literal(
+					error,
+					G_KEY_FILE_ERROR,
+					G_KEY_FILE_ERROR_INVALID_VALUE,
+					"Value of %d for 'max-files' looks implausible");
+			return FALSE;
+		}
+		logger->maxfiles = (guint) tmp_maxfiles;
+
+		if (!check_remaining_keys(key_file, *group, &ierror)) {
+			g_propagate_error(error, ierror);
+			return FALSE;
+		}
+
+		g_key_file_remove_group(key_file, *group, NULL);
+
+		/* insert new logger in list */
+		config->loggers = g_list_append(config->loggers, g_steal_pointer(&logger));
+	}
+
+	return TRUE;
+}
+
 static GHashTable *parse_slots(const char *filename, const char *data_directory, GKeyFile *key_file, GError **error)
 {
 	GError *ierror = NULL;
@@ -394,22 +572,56 @@ static GHashTable *parse_slots(const char *filename, const char *data_directory,
 	return g_steal_pointer(&slots);
 }
 
+void r_config_file_modified_check(void)
+{
+	g_autoptr(GError) ierror = NULL;
+	g_autofree gchar *data = NULL;
+	gsize length;
+	g_autofree gchar *new_checksum = NULL;
+
+	if (!r_context()->config->file_checksum)
+		return;
+
+	if (!g_file_get_contents(r_context()->configpath, &data, &length, &ierror)) {
+		g_warning("Failed to compare config: %s", ierror->message);
+		return;
+	}
+
+	new_checksum = g_compute_checksum_for_data(G_CHECKSUM_SHA256, (guchar*) data, length);
+
+	if (g_strcmp0(r_context()->config->file_checksum, new_checksum) != 0) {
+		g_warning("System configuration file changed on disk! "
+				"Still using old configuration! "
+				"Please restart the rauc service.");
+	}
+}
+
 gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 {
 	GError *ierror = NULL;
+	g_autofree gchar *data = NULL;
+	gsize length;
 	g_autoptr(RaucConfig) c = g_new0(RaucConfig, 1);
 	g_autoptr(GKeyFile) key_file = NULL;
 	gboolean dtbvariant;
 	g_autofree gchar *variant_data = NULL;
 	g_autofree gchar *bundle_formats = NULL;
+	gsize entries;
 
 	g_return_val_if_fail(filename, FALSE);
 	g_return_val_if_fail(config && *config == NULL, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
+	/* We store checksum for later comparison */
+	if (!g_file_get_contents(filename, &data, &length, &ierror)) {
+		g_propagate_error(error, ierror);
+		return FALSE;
+	}
+	c->file_checksum = g_compute_checksum_for_data(G_CHECKSUM_SHA256, (guchar*) data, length);
+
 	key_file = g_key_file_new();
 
-	if (!g_key_file_load_from_file(key_file, filename, G_KEY_FILE_NONE, &ierror)) {
+	if (!g_key_file_load_from_data(key_file, data, length, G_KEY_FILE_NONE, &ierror)) {
 		g_propagate_error(error, ierror);
 		return FALSE;
 	}
@@ -639,7 +851,8 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		if (c->data_directory) {
 			c->statusfile_path = g_build_filename(c->data_directory, "central.raucs", NULL);
 		} else {
-			g_message("Config option 'statusfile=<path>/per-slot' unset, falling back to per-slot status");
+			g_message("No data directory or status file set, falling back to per-slot status.\n"
+					"Consider setting 'data-directory=<path>' or 'statusfile=<path>/per-slot' explicitly.");
 			c->statusfile_path = g_strdup("per-slot");
 		}
 		g_clear_error(&ierror);
@@ -657,7 +870,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 					"Using data-directory= with statusfile=per-slot is not supported.");
 			return FALSE;
 		}
-		g_message("Using per-slot statusfile");
+		g_message("Using per-slot statusfile. System status information not supported!");
 	} else {
 		gchar *resolved = resolve_path(filename, c->statusfile_path);
 		g_free(c->statusfile_path);
@@ -779,6 +992,23 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 	c->streaming_tls_cert = key_file_consume_string(key_file, "streaming", "tls-cert", NULL);
 	c->streaming_tls_key = key_file_consume_string(key_file, "streaming", "tls-key", NULL);
 	c->streaming_tls_ca = key_file_consume_string(key_file, "streaming", "tls-ca", NULL);
+	c->enabled_headers = g_key_file_get_string_list(key_file, "streaming", "send-headers", &entries, &ierror);
+	if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND) ||
+	    g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND)) {
+		g_clear_error(&ierror);
+	} else if (ierror) {
+		g_propagate_error(error, ierror);
+		return FALSE;
+	} else {
+		for (gsize j = 0; j < entries; j++) {
+			if (!r_install_is_supported_http_header(c->enabled_headers[j])) {
+				g_set_error(error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_PARSE,
+						"Automatic HTTP header '%s' not supported", c->enabled_headers[j]);
+				return FALSE;
+			}
+		}
+	}
+	g_key_file_remove_key(key_file, "streaming", "send-headers", NULL);
 	if (!check_remaining_keys(key_file, "streaming", &ierror)) {
 		g_propagate_error(error, ierror);
 		return FALSE;
@@ -820,6 +1050,11 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 	}
 	g_key_file_remove_group(key_file, "handlers", NULL);
 
+	if (!r_event_log_parse_config_sections(key_file, c, &ierror)) {
+		g_propagate_error(error, ierror);
+		return FALSE;
+	}
+
 	/* parse [slot.*.#] sections */
 	c->slots = parse_slots(filename, c->data_directory, key_file, &ierror);
 	if (!c->slots) {
@@ -882,355 +1117,12 @@ void free_config(RaucConfig *config)
 	g_free(config->streaming_tls_cert);
 	g_free(config->streaming_tls_key);
 	g_free(config->streaming_tls_ca);
+	g_strfreev(config->enabled_headers);
 	g_free(config->encryption_key);
 	g_free(config->encryption_cert);
+	g_list_free_full(config->loggers, (GDestroyNotify)r_event_log_free_logger);
 	g_clear_pointer(&config->slots, g_hash_table_destroy);
+	g_free(config->custom_bootloader_backend);
+	g_free(config->file_checksum);
 	g_free(config);
 }
-
-static void status_file_get_slot_status(GKeyFile *key_file, const gchar *group, RaucSlotStatus *slotstatus)
-{
-	GError *ierror = NULL;
-	gchar *digest;
-	guint64 count;
-
-	if (!g_key_file_has_group(key_file, group))
-		g_debug("Group %s not found in key file.", group);
-
-	r_slot_clear_status(slotstatus);
-
-	slotstatus->bundle_compatible = key_file_consume_string(key_file, group, "bundle.compatible", NULL);
-	slotstatus->bundle_version = key_file_consume_string(key_file, group, "bundle.version", NULL);
-	slotstatus->bundle_description = key_file_consume_string(key_file, group, "bundle.description", NULL);
-	slotstatus->bundle_build = key_file_consume_string(key_file, group, "bundle.build", NULL);
-	slotstatus->bundle_hash = key_file_consume_string(key_file, group, "bundle.hash", NULL);
-	slotstatus->status = key_file_consume_string(key_file, group, "status", NULL);
-
-	digest = key_file_consume_string(key_file, group, "sha256", NULL);
-	if (digest) {
-		slotstatus->checksum.type = G_CHECKSUM_SHA256;
-		slotstatus->checksum.digest = digest;
-		slotstatus->checksum.size = g_key_file_get_uint64(key_file, group, "size", NULL);
-	}
-
-	slotstatus->installed_txn = key_file_consume_string(key_file, group, "installed.transaction", NULL);
-	slotstatus->installed_timestamp = key_file_consume_string(key_file, group, "installed.timestamp", NULL);
-	count = g_key_file_get_uint64(key_file, group, "installed.count", &ierror);
-	if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE))
-		g_message("Value of key \"installed.count\" in group [%s] "
-				"is no valid unsigned integer - setting to zero.", group);
-	else if (ierror && !g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND))
-		g_message("Unexpected error while trying to read key \"installed.count\" in group [%s] "
-				"- setting to zero: %s", group, ierror->message);
-	g_clear_error(&ierror);
-	if (count > G_MAXUINT32) {
-		g_message("Value of key \"installed.count\" in group [%s] "
-				"is greater than G_MAXUINT32 - setting to zero.", group);
-		count = 0;
-	}
-	slotstatus->installed_count = count;
-
-	slotstatus->activated_timestamp = key_file_consume_string(key_file, group, "activated.timestamp", NULL);
-	count = g_key_file_get_uint64(key_file, group, "activated.count", &ierror);
-	if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE))
-		g_message("Value of key \"activated.count\" in group [%s] "
-				"is no valid unsigned integer - setting to zero.", group);
-	else if (ierror && !g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND))
-		g_message("Unexpected error while trying to read key \"activated.count\" in group [%s] "
-				"- setting to zero: %s", group, ierror->message);
-	g_clear_error(&ierror);
-	if (count > G_MAXUINT32) {
-		g_message("Value of key \"activated.count\" in group [%s] "
-				"is greater than G_MAXUINT32 - setting to zero.", group);
-		count = 0;
-	}
-	slotstatus->activated_count = count;
-}
-
-static void status_file_set_string_or_remove_key(GKeyFile *key_file, const gchar *group, const gchar *key, gchar *string)
-{
-	if (string)
-		g_key_file_set_string(key_file, group, key, string);
-	else
-		g_key_file_remove_key(key_file, group, key, NULL);
-}
-
-static void status_file_set_slot_status(GKeyFile *key_file, const gchar *group, RaucSlotStatus *slotstatus)
-{
-	status_file_set_string_or_remove_key(key_file, group, "bundle.compatible", slotstatus->bundle_compatible);
-	status_file_set_string_or_remove_key(key_file, group, "bundle.version", slotstatus->bundle_version);
-	status_file_set_string_or_remove_key(key_file, group, "bundle.description", slotstatus->bundle_description);
-	status_file_set_string_or_remove_key(key_file, group, "bundle.build", slotstatus->bundle_build);
-	status_file_set_string_or_remove_key(key_file, group, "bundle.hash", slotstatus->bundle_hash);
-	status_file_set_string_or_remove_key(key_file, group, "status", slotstatus->status);
-
-	if (slotstatus->checksum.digest && slotstatus->checksum.type == G_CHECKSUM_SHA256) {
-		g_key_file_set_string(key_file, group, "sha256", slotstatus->checksum.digest);
-		g_key_file_set_uint64(key_file, group, "size", slotstatus->checksum.size);
-	} else {
-		g_key_file_remove_key(key_file, group, "sha256", NULL);
-		g_key_file_remove_key(key_file, group, "size", NULL);
-	}
-
-	status_file_set_string_or_remove_key(key_file, group, "installed.transaction", slotstatus->installed_txn);
-
-	if (slotstatus->installed_timestamp) {
-		g_key_file_set_string(key_file, group, "installed.timestamp", slotstatus->installed_timestamp);
-	} else {
-		g_key_file_remove_key(key_file, group, "installed.timestamp", NULL);
-	}
-
-	if (slotstatus->installed_count > 0) {
-		g_key_file_set_uint64(key_file, group, "installed.count", slotstatus->installed_count);
-	} else {
-		g_key_file_remove_key(key_file, group, "installed.count", NULL);
-	}
-
-	if (slotstatus->activated_timestamp) {
-		g_key_file_set_string(key_file, group, "activated.timestamp", slotstatus->activated_timestamp);
-	} else {
-		g_key_file_remove_key(key_file, group, "activated.timestamp", NULL);
-	}
-
-	if (slotstatus->activated_count > 0) {
-		g_key_file_set_uint64(key_file, group, "activated.count", slotstatus->activated_count);
-	} else {
-		g_key_file_remove_key(key_file, group, "activated.count", NULL);
-	}
-
-	return;
-}
-
-gboolean read_slot_status(const gchar *filename, RaucSlotStatus *slotstatus, GError **error)
-{
-	GError *ierror = NULL;
-	gboolean res = FALSE;
-	g_autoptr(GKeyFile) key_file = NULL;
-
-	g_return_val_if_fail(filename, FALSE);
-	g_return_val_if_fail(slotstatus, FALSE);
-	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
-
-	key_file = g_key_file_new();
-
-	res = g_key_file_load_from_file(key_file, filename, G_KEY_FILE_NONE, &ierror);
-	if (!res) {
-		g_propagate_error(error, ierror);
-		goto free;
-	}
-
-	status_file_get_slot_status(key_file, "slot", slotstatus);
-
-	res = TRUE;
-free:
-	return res;
-}
-
-gboolean write_slot_status(const gchar *filename, RaucSlotStatus *ss, GError **error)
-{
-	GError *ierror = NULL;
-	g_autoptr(GKeyFile) key_file = NULL;
-	gboolean res = FALSE;
-
-	key_file = g_key_file_new();
-
-	status_file_set_slot_status(key_file, "slot", ss);
-
-	res = g_key_file_save_to_file(key_file, filename, &ierror);
-	if (!res) {
-		g_propagate_error(error, ierror);
-		goto free;
-	}
-
-free:
-	return res;
-}
-
-static void load_slot_status_locally(RaucSlot *dest_slot)
-{
-	GError *ierror = NULL;
-	g_autofree gchar *slotstatuspath = NULL;
-
-	g_return_if_fail(dest_slot);
-
-	if (dest_slot->status)
-		return;
-
-	dest_slot->status = g_new0(RaucSlotStatus, 1);
-
-	if (!r_slot_is_mountable(dest_slot))
-		return;
-
-	/* read slot status */
-	if (!dest_slot->ext_mount_point) {
-		g_message("mounting slot %s", dest_slot->device);
-		if (!r_mount_slot(dest_slot, &ierror)) {
-			g_message("Failed to mount slot %s: %s", dest_slot->device, ierror->message);
-			g_clear_error(&ierror);
-			return;
-		}
-	}
-
-	slotstatuspath = g_build_filename(
-			dest_slot->ext_mount_point ? dest_slot->ext_mount_point : dest_slot->mount_point,
-			"slot.raucs", NULL);
-
-	if (!read_slot_status(slotstatuspath, dest_slot->status, &ierror)) {
-		g_message("Failed to load status file %s: %s", slotstatuspath, ierror->message);
-		g_clear_error(&ierror);
-	}
-
-	if (!dest_slot->ext_mount_point) {
-		if (!r_umount_slot(dest_slot, &ierror)) {
-			g_message("Failed to unmount slot %s: %s", dest_slot->device, ierror->message);
-			g_clear_error(&ierror);
-			return;
-		}
-	}
-}
-
-static void load_slot_status_globally(void)
-{
-	GError *ierror = NULL;
-	GHashTable *slots = r_context()->config->slots;
-	g_autoptr(GKeyFile) key_file = g_key_file_new();
-	g_auto(GStrv) groups = NULL;
-	gchar **group, *slotname;
-	GHashTableIter iter;
-	RaucSlot *slot;
-
-	g_return_if_fail(r_context()->config->statusfile_path);
-
-	g_key_file_load_from_file(key_file, r_context()->config->statusfile_path, G_KEY_FILE_NONE, &ierror);
-	if (ierror && !g_error_matches(ierror, G_FILE_ERROR, G_FILE_ERROR_NOENT))
-		g_message("load_slot_status_globally: %s.", ierror->message);
-	g_clear_error(&ierror);
-
-	/* Load all slot states included in the statusfile */
-	groups = g_key_file_get_groups(key_file, NULL);
-	for (group = groups; *group != NULL; group++) {
-		if (!g_str_has_prefix(*group, RAUC_SLOT_PREFIX "."))
-			continue;
-
-		slotname = *group + strlen(RAUC_SLOT_PREFIX ".");
-		slot = g_hash_table_lookup(slots, slotname);
-		if (!slot || slot->status)
-			continue;
-
-		slot->status = g_new0(RaucSlotStatus, 1);
-		g_debug("Load status for slot %s.", slot->name);
-		status_file_get_slot_status(key_file, *group, slot->status);
-	}
-
-	/* Set all other slots to the default status */
-	g_hash_table_iter_init(&iter, slots);
-	while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &slot)) {
-		if (slot->status)
-			continue;
-
-		g_debug("Set default status for slot %s.", slot->name);
-		slot->status = g_new0(RaucSlotStatus, 1);
-	}
-}
-
-void load_slot_status(RaucSlot *dest_slot)
-{
-	g_return_if_fail(dest_slot);
-
-	if (!dest_slot->status) {
-		if (g_strcmp0(r_context()->config->statusfile_path, "per-slot") == 0)
-			load_slot_status_locally(dest_slot);
-		else
-			load_slot_status_globally();
-	}
-
-	r_slot_clean_data_directory(dest_slot);
-}
-
-static gboolean save_slot_status_locally(RaucSlot *dest_slot, GError **error)
-{
-	GError *ierror = NULL;
-	gboolean res = FALSE;
-	g_autofree gchar *slotstatuspath = NULL;
-
-	g_return_val_if_fail(dest_slot, FALSE);
-	g_return_val_if_fail(dest_slot->status, FALSE);
-	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
-
-	if (!r_slot_is_mountable(dest_slot)) {
-		res = TRUE;
-		goto free;
-	}
-
-	g_debug("mounting slot %s", dest_slot->device);
-	res = r_mount_slot(dest_slot, &ierror);
-	if (!res) {
-		g_propagate_error(error, ierror);
-		goto free;
-	}
-
-	slotstatuspath = g_build_filename(dest_slot->mount_point, "slot.raucs", NULL);
-	g_message("Updating slot file %s", slotstatuspath);
-
-	res = write_slot_status(slotstatuspath, dest_slot->status, &ierror);
-	if (!res) {
-		g_propagate_error(error, ierror);
-		r_umount_slot(dest_slot, NULL);
-
-		goto free;
-	}
-
-	res = r_umount_slot(dest_slot, &ierror);
-	if (!res) {
-		g_propagate_error(error, ierror);
-		goto free;
-	}
-
-free:
-	return res;
-}
-
-static gboolean save_slot_status_globally(GError **error)
-{
-	g_autoptr(GKeyFile) key_file = g_key_file_new();
-	GError *ierror = NULL;
-	GHashTableIter iter;
-	RaucSlot *slot;
-	gboolean res;
-
-	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
-	g_return_val_if_fail(r_context()->config->statusfile_path, FALSE);
-
-	g_debug("Saving global slot status");
-
-	/* Save all slot status information */
-	g_hash_table_iter_init(&iter, r_context()->config->slots);
-	while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &slot)) {
-		g_autofree gchar *group = NULL;
-
-		if (!slot->status) {
-			continue;
-		}
-		group = g_strdup_printf(RAUC_SLOT_PREFIX ".%s", slot->name);
-		status_file_set_slot_status(key_file, group, slot->status);
-	}
-
-	res = g_key_file_save_to_file(key_file, r_context()->config->statusfile_path, &ierror);
-	if (!res)
-		g_propagate_error(error, ierror);
-
-	return res;
-}
-
-gboolean save_slot_status(RaucSlot *dest_slot, GError **error)
-{
-	g_return_val_if_fail(dest_slot, FALSE);
-	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
-
-	r_slot_clean_data_directory(dest_slot);
-
-	if (g_strcmp0(r_context()->config->statusfile_path, "per-slot") == 0)
-		return save_slot_status_locally(dest_slot, error);
-	else
-		return save_slot_status_globally(error);
-}
