@@ -15,12 +15,14 @@
 #include "bootchooser.h"
 #include "bundle.h"
 #include "context.h"
+#include "event_log.h"
 #include "install.h"
 #include "manifest.h"
 #include "mark.h"
 #include "mount.h"
 #include "service.h"
 #include "signature.h"
+#include "status_file.h"
 #include "update_handler.h"
 #include "utils.h"
 
@@ -839,20 +841,6 @@ static gboolean launch_and_wait_custom_handler(RaucInstallArgs *args, gchar* bun
 
 	r_context_begin_step_weighted("launch_and_wait_custom_handler", "Launching update handler", 0, 6);
 
-	/* Allow overriding compatible check by hook */
-	if (manifest->hooks.install_check) {
-		run_bundle_hook(manifest, bundledir, "install-check", &ierror);
-		if (ierror) {
-			g_propagate_error(error, ierror);
-			res = FALSE;
-			goto out;
-		}
-	} else if (!verify_compatible(args, manifest, &ierror)) {
-		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto out;
-	}
-
 	handler_name = g_build_filename(bundledir, manifest->handler_name, NULL);
 	handler_args = g_ptr_array_new_full(0, g_free);
 	if (manifest->handler_args) {
@@ -948,20 +936,41 @@ skip_filename_checks:
 	return TRUE;
 }
 
+static void update_slot_status(RaucSlotStatus *slot_state, const gchar* status, const RaucManifest *manifest, const RImageInstallPlan *plan, const RaucInstallArgs *args)
+{
+	g_autoptr(GDateTime) now = NULL;
+
+	r_slot_clear_status(slot_state);
+
+	now = g_date_time_new_now_utc();
+
+	slot_state->bundle_compatible = g_strdup(manifest->update_compatible);
+	slot_state->bundle_version = g_strdup(manifest->update_version);
+	slot_state->bundle_description = g_strdup(manifest->update_description);
+	slot_state->bundle_build = g_strdup(manifest->update_build);
+	slot_state->bundle_hash = g_strdup(manifest->hash);
+	slot_state->status = g_strdup(status);
+	slot_state->checksum.type = plan->image->checksum.type;
+	slot_state->checksum.digest = g_strdup(plan->image->checksum.digest);
+	slot_state->checksum.size = plan->image->checksum.size;
+	slot_state->installed_txn = g_strdup(args->transaction);
+	slot_state->installed_timestamp = g_date_time_format(now, "%Y-%m-%dT%H:%M:%SZ");
+	slot_state->installed_count++;
+}
+
 static gboolean handle_slot_install_plan(const RaucManifest *manifest, const RImageInstallPlan *plan, RaucInstallArgs *args, const char *hook_name, GError **error)
 {
 	GError *ierror = NULL;
 	RaucSlotStatus *slot_state = NULL;
-	g_autoptr(GDateTime) now = NULL;
 
 	install_args_update(args, "Checking slot %s", plan->target_slot->name);
 
 	r_context_begin_step_weighted_formatted("check_slot", 0, 1, "Checking slot %s", plan->target_slot->name);
 
-	load_slot_status(plan->target_slot);
+	r_slot_status_load(plan->target_slot);
 	slot_state = plan->target_slot->status;
 
-	/* In case we failed unmounting while reading status
+	/* In case we failed unmounting while reading per-slot status
 	 * file, abort here */
 	if (plan->target_slot->mount_point) {
 		g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_MOUNTED,
@@ -970,6 +979,22 @@ static gboolean handle_slot_install_plan(const RaucManifest *manifest, const RIm
 		return FALSE;
 	}
 
+	/* For global slot status: Clear checksum info and make status
+	 * 'pending' to prevent the slot status from looking valid later in
+	 * case we crash while installing. */
+	if (g_strcmp0(r_context()->config->statusfile_path, "per-slot") != 0) {
+		g_clear_pointer(&slot_state->status, g_free);
+		slot_state->status = g_strdup("pending");
+		g_clear_pointer(&slot_state->checksum.digest, g_free);
+		slot_state->checksum.size = 0;
+
+		if (!r_slot_status_save(plan->target_slot, &ierror)) {
+			g_propagate_prefixed_error(error, ierror, "Error while writing status file: ");
+			r_context_end_step("check_slot", FALSE);
+			return FALSE;
+		}
+	}
+
 	/* if explicitly enabled, skip update of up-to-date slots */
 	if (!plan->target_slot->install_same && g_strcmp0(plan->image->checksum.digest, slot_state->checksum.digest) == 0) {
 		install_args_update(args, "Skipping update for correct image %s", plan->image->filename);
@@ -978,6 +1003,16 @@ static gboolean handle_slot_install_plan(const RaucManifest *manifest, const RIm
 
 		/* Dummy step to indicate slot was skipped */
 		r_context_begin_step("skip_image", "Copying image skipped", 0);
+
+		/* Update the status also for skipped slots */
+		g_message("Updating slot %s status", plan->target_slot->name);
+		update_slot_status(slot_state, "ok", manifest, plan, args);
+		if (!r_slot_status_save(plan->target_slot, &ierror)) {
+			g_propagate_prefixed_error(error, ierror, "Error while writing status file: ");
+			r_context_end_step("skip_image", FALSE);
+			return FALSE;
+		}
+
 		r_context_end_step("skip_image", TRUE);
 
 		install_args_update(args, "Updating slot %s done", plan->target_slot->name);
@@ -990,6 +1025,7 @@ static gboolean handle_slot_install_plan(const RaucManifest *manifest, const RIm
 	r_context_end_step("check_slot", TRUE);
 
 	install_args_update(args, "Updating slot %s", plan->target_slot->name);
+	r_event_log_message(R_EVENT_LOG_TYPE_WRITE_SLOT, "Updating slot %s", plan->target_slot->name);
 
 	/* update slot */
 	if (plan->image->hooks.install) {
@@ -1004,33 +1040,26 @@ static gboolean handle_slot_install_plan(const RaucManifest *manifest, const RIm
 	r_context_begin_step_weighted_formatted("copy_image", 0, 9, "Copying image to %s", plan->target_slot->name);
 
 	if (!plan->slot_handler(plan->image, plan->target_slot, hook_name, &ierror)) {
+		g_autoptr(GError) ierror_status = NULL;
+
 		g_propagate_prefixed_error(error, ierror,
 				"Failed updating slot %s: ", plan->target_slot->name);
 		r_context_end_step("copy_image", FALSE);
+
+		g_message("Updating slot %s status", plan->target_slot->name);
+		update_slot_status(slot_state, "failed", manifest, plan, args);
+		if (!r_slot_status_save(plan->target_slot, &ierror_status)) {
+			g_warning("Error while writing status file after slot update failure: %s", ierror_status->message);
+		}
+
 		return FALSE;
 	}
 
-	r_slot_clear_status(slot_state);
-
-	now = g_date_time_new_now_utc();
-
-	slot_state->bundle_compatible = g_strdup(manifest->update_compatible);
-	slot_state->bundle_version = g_strdup(manifest->update_version);
-	slot_state->bundle_description = g_strdup(manifest->update_description);
-	slot_state->bundle_build = g_strdup(manifest->update_build);
-	slot_state->bundle_hash = g_strdup(manifest->hash);
-	slot_state->status = g_strdup("ok");
-	slot_state->checksum.type = plan->image->checksum.type;
-	slot_state->checksum.digest = g_strdup(plan->image->checksum.digest);
-	slot_state->checksum.size = plan->image->checksum.size;
-	slot_state->installed_txn = g_strdup(args->transaction);
-	slot_state->installed_timestamp = g_date_time_format(now, "%Y-%m-%dT%H:%M:%SZ");
-	slot_state->installed_count++;
-
 	r_context_end_step("copy_image", TRUE);
 
-	install_args_update(args, "Updating slot %s status", plan->target_slot->name);
-	if (!save_slot_status(plan->target_slot, &ierror)) {
+	g_message("Updating slot %s status", plan->target_slot->name);
+	update_slot_status(slot_state, "ok", manifest, plan, args);
+	if (!r_slot_status_save(plan->target_slot, &ierror)) {
 		g_propagate_prefixed_error(error, ierror, "Error while writing status file: ");
 		return FALSE;
 	}
@@ -1070,6 +1099,66 @@ static RaucSlot* get_boot_mark_slot(const GPtrArray *install_plans)
 	return bootslot;
 }
 
+#define MESSAGE_ID_INSTALLATION_STARTED   "b05410e8a93345389cd061aab1e9516d"
+#define MESSAGE_ID_INSTALLATION_SUCCEEDED "0163db5468ac4237b090d28490c301ed"
+#define MESSAGE_ID_INSTALLATION_FAILED    "c48141f7fd49443aafff862b4809168f"
+#define MESSAGE_ID_INSTALLATION_REJECTED  "60bea7e4fea549ccad68af457308b13a"
+
+static void log_event_installation_started(RaucInstallArgs *args)
+{
+	g_log_structured(R_EVENT_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
+			"RAUC_EVENT_TYPE", "install",
+			"MESSAGE_ID", MESSAGE_ID_INSTALLATION_STARTED,
+			"TRANSACTION_ID", args->transaction,
+			"MESSAGE", "Installation %.8s started", args->transaction // truncate ID for readability
+			);
+}
+
+/**
+ * @param args RaucInstallArgs
+ * @param manifest Manifest
+ * @param GError Error or NULL
+ */
+static void log_event_installation_done(RaucInstallArgs *args, RaucManifest *manifest, const GError *error)
+{
+	g_autofree gchar *formatted = NULL;
+	GLogField fields[] = {
+		{"MESSAGE", NULL, -1 },
+		{"MESSAGE_ID", NULL, -1 },
+		{"GLIB_DOMAIN", R_EVENT_LOG_DOMAIN, -1},
+		{"RAUC_EVENT_TYPE", "install", -1},
+		{"BUNDLE_HASH", "", -1},
+		{"BUNDLE_DESCRIPTION", "", -1},
+		{"BUNDLE_VERSION", "", -1},
+		{"TRANSACTION_ID", args->transaction, -1},
+	};
+
+	g_return_if_fail(args);
+
+	if (error) {
+		if (g_error_matches(error, R_INSTALL_ERROR, R_INSTALL_ERROR_REJECTED) ||
+		    g_error_matches(error, R_INSTALL_ERROR, R_INSTALL_ERROR_COMPAT_MISMATCH)) {
+			formatted = g_strdup_printf("Installation %.8s rejected: %s", args->transaction, error->message);
+			fields[1].value = MESSAGE_ID_INSTALLATION_REJECTED;
+		} else {
+			formatted = g_strdup_printf("Installation %.8s failed: %s", args->transaction, error->message);
+			fields[1].value = MESSAGE_ID_INSTALLATION_FAILED;
+		}
+	} else {
+		formatted = g_strdup_printf("Installation %.8s succeeded", args->transaction);
+		fields[1].value = MESSAGE_ID_INSTALLATION_SUCCEEDED;
+	}
+
+	fields[0].value = formatted;
+	if (manifest) {
+		fields[4].value = manifest->hash ?: "";
+		fields[5].value = manifest->update_description ?: "";
+		fields[6].value = manifest->update_version ?: "";
+	}
+
+	g_log_structured_array(G_LOG_LEVEL_MESSAGE, fields, G_N_ELEMENTS(fields));
+}
+
 static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bundledir, RaucManifest *manifest, GHashTable *target_group, GError **error)
 {
 	g_autofree gchar *hook_name = NULL;
@@ -1085,28 +1174,6 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 
 	boot_mark_slot = get_boot_mark_slot(install_plans);
 
-	/* Allow overriding compatible check by hook */
-	if (manifest->hooks.install_check) {
-		run_bundle_hook(manifest, bundledir, "install-check", &ierror);
-		if (ierror) {
-			if (g_error_matches(ierror, R_INSTALL_ERROR, R_INSTALL_ERROR_REJECTED)) {
-				g_propagate_prefixed_error(
-						error,
-						ierror,
-						"Bundle rejected: ");
-			} else {
-				g_propagate_prefixed_error(
-						error,
-						ierror,
-						"Install-check hook failed: ");
-			}
-			return FALSE;
-		}
-	} else if (!verify_compatible(args, manifest, &ierror)) {
-		g_propagate_error(error, ierror);
-		return FALSE;
-	}
-
 	if (!pre_install_checks(bundledir, install_plans, target_group, &ierror)) {
 		g_propagate_error(error, ierror);
 		return FALSE;
@@ -1115,7 +1182,7 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 	if (boot_mark_slot) {
 		/* Mark boot slot non-bootable */
 		g_message("Marking target slot %s as non-bootable...", boot_mark_slot->name);
-		if (!r_boot_set_state(boot_mark_slot, FALSE, &ierror)) {
+		if (!r_mark_bad(boot_mark_slot, &ierror)) {
 			g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_MARK_NONBOOTABLE,
 					"Failed marking slot %s non-bootable: %s", boot_mark_slot->name, ierror->message);
 			g_clear_error(&ierror);
@@ -1146,7 +1213,7 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 		if (r_context()->config->activate_installed) {
 			/* Mark boot slot bootable */
 			g_message("Marking target slot %s as bootable...", boot_mark_slot->name);
-			if (!mark_active(boot_mark_slot, &ierror)) {
+			if (!r_mark_active(boot_mark_slot, &ierror)) {
 				g_propagate_prefixed_error(error, ierror,
 						"Failed marking slot %s bootable: ", boot_mark_slot->name);
 				return FALSE;
@@ -1160,6 +1227,94 @@ static gboolean launch_and_wait_default_handler(RaucInstallArgs *args, gchar* bu
 	return TRUE;
 }
 
+static gchar* get_uptime(void)
+{
+	g_autofree gchar *contents = NULL;
+	g_autoptr(GError) ierror = NULL;
+	g_auto(GStrv) uptime = NULL;
+
+	if (!g_file_get_contents("/proc/uptime", &contents, NULL, &ierror)) {
+		g_warning("Failed to get uptime: %s", ierror->message);
+		return NULL;
+	}
+
+	/* file contains two values and a newline, 'chomp' in-place and split then */
+	uptime = g_strsplit(g_strchomp(contents), " ", 2);
+
+	return g_strdup(uptime[0]);
+}
+
+/* If the input key starts with RAUC_HTTP_, it returns a valid HTTP header
+ * string with 'RAUC_HTTP_' replaced by 'RAUC-'.
+ * If the input string does not start with RAUC_HTTP_, NULL is returned.
+ */
+static gchar *system_info_to_header(const gchar *key, const gchar *value)
+{
+	g_autofree gchar *header_key = NULL;
+
+	g_return_val_if_fail(key, NULL);
+	g_return_val_if_fail(value, NULL);
+
+	if (!g_str_has_prefix(key, "RAUC_HTTP_"))
+		return NULL;
+
+	header_key = g_strdup(key + strlen("RAUC_HTTP_"));
+	for (size_t i = 0; i < strlen(header_key); i++) {
+		if (header_key[i] == '_')
+			header_key[i] = '-';
+	}
+
+	return g_strdup_printf("RAUC-%s: %s", header_key, value);
+}
+
+static gchar **assemble_info_headers(RaucInstallArgs *args)
+{
+	GPtrArray *headers = g_ptr_array_new_with_free_func(g_free);
+
+	g_return_val_if_fail(args, NULL);
+
+	if (!r_context()->config->enabled_headers)
+		goto no_std_headers;
+
+	for (gchar **header = r_context()->config->enabled_headers; *header; header++) {
+		/* Add static system information */
+		if (g_strcmp0(*header, "boot-id") == 0)
+			g_ptr_array_add(headers, g_strdup_printf("RAUC-Boot-ID: %s", r_context()->boot_id));
+		if (g_strcmp0(*header, "machine-id") == 0)
+			g_ptr_array_add(headers, g_strdup_printf("RAUC-Machine-ID: %s", r_context()->machine_id));
+		if (g_strcmp0(*header, "serial") == 0)
+			g_ptr_array_add(headers, g_strdup_printf("RAUC-Serial: %s", r_context()->system_serial));
+		if (g_strcmp0(*header, "variant") == 0)
+			g_ptr_array_add(headers, g_strdup_printf("RAUC-Variant: %s", r_context()->config->system_variant));
+		/* Add per-installation information */
+		if (g_strcmp0(*header, "transaction-id") == 0)
+			g_ptr_array_add(headers, g_strdup_printf("RAUC-Transaction-ID: %s", args->transaction));
+		/* Add live information */
+		if (g_strcmp0(*header, "uptime") == 0) {
+			g_autofree gchar *uptime = get_uptime();
+			g_ptr_array_add(headers, g_strdup_printf("RAUC-Uptime: %s", uptime));
+		}
+	}
+
+no_std_headers:
+
+	if (r_context()->system_info) {
+		GHashTableIter iter;
+		gchar *key = NULL;
+		gchar *value = NULL;
+
+		g_hash_table_iter_init(&iter, r_context()->system_info);
+		while (g_hash_table_iter_next(&iter, (gpointer*) &key, (gpointer*) &value)) {
+			gchar *header = system_info_to_header(key, value);
+			if (header)
+				g_ptr_array_add(headers, header);
+		}
+	}
+	g_ptr_array_add(headers, NULL);
+
+	return (gchar**) g_ptr_array_free(headers, FALSE);
+}
+
 gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 {
 	const gchar* bundlefile = args->name;
@@ -1178,7 +1333,7 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 
 	r_context_begin_step("do_install_bundle", "Installing", 10);
 
-	g_message("Installation %s started", args->transaction);
+	log_event_installation_started(args);
 
 	r_context_begin_step("determine_slot_states", "Determining slot states", 0);
 	res = update_external_mount_points(&ierror);
@@ -1191,6 +1346,8 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 	// TODO: mount info in context ?
 	install_args_update(args, "Checking and mounting bundle...");
 
+	args->access_args.http_info_headers = assemble_info_headers(args);
+
 	res = check_bundle(bundlefile, &bundle, CHECK_BUNDLE_DEFAULT, &args->access_args, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
@@ -1198,6 +1355,7 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 	}
 
 	if (bundle->manifest && bundle->manifest->bundle_format == R_MANIFEST_FORMAT_CRYPT && !bundle->was_encrypted) {
+		r_event_log_message(R_EVENT_LOG_TYPE_INSTALL, "Installation %.8s rejected: Refusing to install unencrypted crypt bundles", args->transaction);
 		g_set_error(error, R_INSTALL_ERROR, R_INSTALL_ERROR_REJECTED, "Refusing to install unencrypted crypt bundles");
 		res = FALSE;
 		goto out;
@@ -1222,6 +1380,7 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 	}
 
 	handler_env = prepare_environment(bundle->mount_point, bundle->manifest, target_group);
+	handler_env = g_environ_setenv(handler_env, "RAUC_TRANSACTION_ID", args->transaction, TRUE);
 
 	if (r_context()->config->preinstall_handler) {
 		g_message("Starting pre install handler: %s", r_context()->config->preinstall_handler);
@@ -1232,6 +1391,30 @@ gboolean do_install_bundle(RaucInstallArgs *args, GError **error)
 		}
 	}
 
+	/* Allow overriding compatible check by hook */
+	if (bundle->manifest->hooks.install_check) {
+		run_bundle_hook(bundle->manifest, bundle->mount_point, "install-check", &ierror);
+		if (ierror) {
+			res = FALSE;
+			if (g_error_matches(ierror, R_INSTALL_ERROR, R_INSTALL_ERROR_REJECTED)) {
+				g_propagate_prefixed_error(
+						error,
+						ierror,
+						"Bundle rejected: ");
+			} else {
+				g_propagate_prefixed_error(
+						error,
+						ierror,
+						"Install-check hook failed: ");
+			}
+			goto umount;
+		}
+	} else if (!verify_compatible(args, bundle->manifest, &ierror)) {
+		res = FALSE;
+		g_propagate_error(error, ierror);
+		goto umount;
+	}
+
 	if (bundle->manifest->handler_name) {
 		g_message("Using custom handler: %s", bundle->manifest->handler_name);
 		res = launch_and_wait_custom_handler(args, bundle->mount_point, bundle->manifest, target_group, &ierror);
@@ -1264,6 +1447,8 @@ umount:
 	r_context()->install_info->mounted_bundle = NULL;
 
 out:
+	log_event_installation_done(args, bundle ? bundle->manifest : NULL, error ? *error : NULL);
+
 	r_context_end_step("do_install_bundle", res);
 
 	return res;
@@ -1346,3 +1531,10 @@ gboolean install_run(RaucInstallArgs *args)
 
 	return TRUE;
 }
+
+static const gchar *supported_http_headers[] = {"boot-id", "transaction-id", "machine-id", "serial", "variant", "uptime", NULL};
+
+gboolean r_install_is_supported_http_header(const gchar *header)
+{
+	return g_strv_contains(supported_http_headers, header);
+}
