@@ -16,10 +16,12 @@
 #include "bootchooser.h"
 #include "config_file.h"
 #include "context.h"
+#include "event_log.h"
 #include "install.h"
 #include "rauc-installer-generated.h"
 #include "service.h"
 #include "signature.h"
+#include "status_file.h"
 #include "update_handler.h"
 #include "utils.h"
 #include "mark.h"
@@ -41,6 +43,7 @@ gchar **intermediate = NULL;
 gchar *signing_keyring = NULL;
 gchar *mksquashfs_args = NULL;
 gchar *casync_args = NULL;
+gchar **convert_ignore_images = NULL;
 gchar **recipients = NULL;
 gchar *handler_args = NULL;
 gchar *bootslot = NULL;
@@ -528,9 +531,8 @@ static gboolean resign_start(int argc, char **argv)
 	}
 
 	if (r_context()->certpath == NULL ||
-	    r_context()->keypath == NULL ||
-	    r_context()->keyringpath == NULL) {
-		g_printerr("Cert, key and keyring files must be provided\n");
+	    r_context()->keypath == NULL) {
+		g_printerr("Cert and key files must be provided\n");
 		r_exit_status = 1;
 		goto out;
 	}
@@ -764,7 +766,7 @@ static gboolean convert_start(int argc, char **argv)
 		goto out;
 	}
 
-	if (!create_casync_bundle(bundle, argv[3], &ierror)) {
+	if (!create_casync_bundle(bundle, argv[3], (const gchar**) convert_ignore_images, &ierror)) {
 		g_printerr("Failed to create bundle: %s\n", ierror->message);
 		g_clear_error(&ierror);
 		r_exit_status = 1;
@@ -1400,7 +1402,10 @@ static gchar* r_status_formatter_readable(RaucStatusPrint *status)
 	g_string_append_printf(text, "Booted from: %s (%s)\n\n", bootedfrom ? bootedfrom->name : NULL, status->bootslot);
 
 	g_string_append(text, "=== Bootloader ===\n");
-	g_string_append_printf(text, "Activated: %s (%s)\n\n", status->primary ? status->primary->name : NULL, status->primary ? status->primary->bootname : NULL);
+	if (!status->primary)
+		g_string_append_printf(text, "Activated: none\n\n");
+	else
+		g_string_append_printf(text, "Activated: %s (%s)\n\n", status->primary->name, status->primary->bootname);
 
 	g_string_append(text, "=== Slot States ===\n");
 	slotclasses = r_slot_get_root_classes(status->slots);
@@ -1449,7 +1454,7 @@ static gchar* r_status_formatter_shell(RaucStatusPrint *status)
 	formatter_shell_append(text, "RAUC_BOOT_PRIMARY", status->primary ? status->primary->name : NULL);
 
 	slotnames = g_ptr_array_new();
-	slotnumbers = g_ptr_array_new();
+	slotnumbers = g_ptr_array_new_with_free_func(g_free);
 	g_hash_table_iter_init(&iter, status->slots);
 	while (g_hash_table_iter_next(&iter, (gpointer*) &name, NULL)) {
 		g_ptr_array_add(slotnames, name);
@@ -1646,9 +1651,7 @@ static gchar* r_status_formatter_json(RaucStatusPrint *status, gboolean pretty)
 static RaucSlotStatus* r_variant_get_slot_state(GVariant *vardict)
 {
 	RaucSlotStatus *slot_state = g_new0(RaucSlotStatus, 1);
-	GVariantDict dict;
-
-	g_variant_dict_init(&dict, vardict);
+	g_auto(GVariantDict) dict = G_VARIANT_DICT_INIT(vardict);
 
 	g_variant_dict_lookup(&dict, "bundle.compatible", "s", &slot_state->bundle_compatible);
 	g_variant_dict_lookup(&dict, "bundle.version", "s", &slot_state->bundle_version);
@@ -1665,9 +1668,6 @@ static RaucSlotStatus* r_variant_get_slot_state(GVariant *vardict)
 	g_variant_dict_lookup(&dict, "activated.timestamp", "s", &slot_state->activated_timestamp);
 	g_variant_dict_lookup(&dict, "activated.count", "u", &slot_state->activated_count);
 
-	vardict = g_variant_dict_end(&dict);
-	g_variant_unref(vardict);
-
 	return slot_state;
 }
 
@@ -1731,6 +1731,7 @@ static gboolean retrieve_slot_states_via_dbus(GHashTable **slots, GError **error
 		g_autofree gchar *parent = NULL;
 		g_autofree gchar *state = NULL;
 		g_autofree gchar *boot_good = NULL;
+		g_autofree gchar *sclass = NULL;
 
 		/* if already existing, skip */
 		if (g_hash_table_lookup(*slots, slot_name)) {
@@ -1740,9 +1741,10 @@ static gboolean retrieve_slot_states_via_dbus(GHashTable **slots, GError **error
 
 		/* Create slot struct and fill up with information */
 		slot = g_new0(RaucSlot, 1);
-		slot->name = g_strdup(slot_name);
+		slot->name = g_intern_string(slot_name);
 		g_variant_dict_init(&dict, vardict);
-		g_variant_dict_lookup(&dict, "class", "s", &slot->sclass);
+		g_variant_dict_lookup(&dict, "class", "s", &sclass);
+		slot->sclass = g_intern_string(sclass);
 		g_variant_dict_lookup(&dict, "device", "s", &slot->device);
 		g_variant_dict_lookup(&dict, "type", "s", &slot->type);
 		g_variant_dict_lookup(&dict, "bootname", "s", &slot->bootname);
@@ -1755,7 +1757,7 @@ static gboolean retrieve_slot_states_via_dbus(GHashTable **slots, GError **error
 			 * can replace with a pointer to the real one
 			 * afterwards */
 			slot->parent = g_new0(RaucSlot, 1);
-			slot->parent->name = g_steal_pointer(&parent);
+			slot->parent->name = g_intern_string(parent);
 		}
 		g_variant_dict_lookup(&dict, "mountpoint", "s", &slot->mount_point);
 		g_variant_dict_lookup(&dict, "boot-status", "s", &boot_good);
@@ -1769,6 +1771,7 @@ static gboolean retrieve_slot_states_via_dbus(GHashTable **slots, GError **error
 			slot->status = r_variant_get_slot_state(vardict);
 		}
 		g_hash_table_insert(*slots, (gchar*)slot->name, slot);
+		g_variant_dict_clear(&dict);
 	}
 
 	/* Now we replace the dummy parent slots with the pointer to the real
@@ -1808,7 +1811,7 @@ static gboolean retrieve_status_via_dbus(RaucStatusPrint **status_print, GError
 	GError *ierror = NULL;
 	RInstaller *proxy;
 	g_autoptr(RaucStatusPrint) istatus = NULL;
-	gchar *primary = NULL;
+	g_autofree gchar *primary = NULL;
 
 	g_return_val_if_fail(status_print != NULL && *status_print == NULL, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
@@ -1911,7 +1914,7 @@ static gboolean status_start(int argc, char **argv)
 
 			g_hash_table_iter_init(&iter, r_context()->config->slots);
 			while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &slot))
-				load_slot_status(slot);
+				r_slot_status_load(slot);
 		}
 
 		status_print = g_new0(RaucStatusPrint, 1);
@@ -2000,6 +2003,60 @@ static gboolean status_start(int argc, char **argv)
 	return TRUE;
 }
 
+#define MESSAGE_ID_BOOTED "e60e0addd3454cb8b796eae0d497af96"
+
+static void r_event_log_booted(const RaucSlot *booted_slot)
+{
+	g_autofree gchar *message = NULL;
+	GLogField fields[] = {
+		{"MESSAGE", NULL, -1 },
+		{"MESSAGE_ID", MESSAGE_ID_BOOTED, -1 },
+		{"GLIB_DOMAIN", R_EVENT_LOG_DOMAIN, -1},
+		{"RAUC_EVENT_TYPE", "boot", -1},
+		{"BOOT_ID", NULL, -1},
+		{"BUNDLE_HASH", NULL, -1},
+	};
+
+	g_return_if_fail(booted_slot);
+
+	message = g_strdup_printf("Booted into %s (%s)", booted_slot->name, booted_slot->bootname);
+	fields[0].value = message;
+	fields[4].value = r_context()->boot_id;
+	if (booted_slot->status && booted_slot->status->bundle_hash) {
+		fields[5].value =  booted_slot->status->bundle_hash;
+	} else {
+		fields[5].value =  "unknown";
+	}
+	g_log_structured_array(G_LOG_LEVEL_INFO, fields, G_N_ELEMENTS(fields));
+}
+
+G_GNUC_UNUSED
+static void create_run_links(void)
+{
+	g_autoptr(GError) ierror = NULL;
+	GHashTableIter iter;
+	RaucSlot *slot;
+
+	if (g_mkdir_with_parents("/run/rauc/slots/active", 0755) != 0) {
+		g_warning("Failed to create /run/rauc/slots/active");
+		return;
+	}
+
+	g_hash_table_iter_init(&iter, r_context()->config->slots);
+	while (g_hash_table_iter_next(&iter, NULL, (gpointer*) &slot)) {
+		g_autofree gchar* path = NULL;
+
+		if (!(slot->state & ST_ACTIVE))
+			continue;
+
+		path = g_build_filename("/run/rauc/slots/active", slot->sclass, NULL);
+
+		if (!r_update_symlink(slot->device, path, &ierror)) {
+			g_warning("Failed to create symlink for active slot: %s", ierror->message);
+		}
+	}
+}
+
 G_GNUC_UNUSED
 static gboolean service_start(int argc, char **argv)
 {
@@ -2013,6 +2070,35 @@ static gboolean service_start(int argc, char **argv)
 		return TRUE;
 	}
 
+	create_run_links();
+
+	if (r_context()->system_status) {
+		/* Boot ID-based system reboot vs service restart detection */
+		if (g_strcmp0(r_context()->system_status->boot_id, r_context()->boot_id) == 0) {
+			g_message("Restarted RAUC service");
+			r_event_log_message(R_EVENT_LOG_TYPE_SERVICE, "Service restarted");
+		} else {
+			RaucSlot *booted_slot = r_slot_find_by_device(r_context()->config->slots, r_context()->bootslot);
+			if (!booted_slot)
+				booted_slot = r_slot_find_by_bootname(r_context()->config->slots, r_context()->bootslot);
+			r_slot_status_load(booted_slot);
+
+			r_event_log_booted(booted_slot);
+
+			/* update boot ID */
+			g_free(r_context()->system_status->boot_id);
+			r_context()->system_status->boot_id = g_strdup(r_context()->boot_id);
+
+			if (!r_system_status_save(&ierror)) {
+				g_warning("Failed to save system status: %s", ierror->message);
+				g_clear_error(&ierror);
+			}
+		}
+	} else {
+		g_message("Started RAUC service");
+		r_event_log_message(R_EVENT_LOG_TYPE_SERVICE, "Service started");
+	}
+
 	r_exit_status = r_service_run() ? 0 : 1;
 
 	return TRUE;
@@ -2144,6 +2230,7 @@ static GOptionEntry entries_convert[] = {
 	{"signing-keyring", '\0', 0, G_OPTION_ARG_FILENAME, &signing_keyring, "verification keyring file", "PEMFILE"},
 	{"mksquashfs-args", '\0', 0, G_OPTION_ARG_STRING, &mksquashfs_args, "mksquashfs extra args", "ARGS"},
 	{"casync-args", '\0', 0, G_OPTION_ARG_STRING, &casync_args, "casync extra args", "ARGS"},
+	{"ignore-image", '\0', 0, G_OPTION_ARG_STRING_ARRAY, &convert_ignore_images, "ignore image during conversion", "SLOTCLASS"},
 	{0}
 };
 
@@ -2163,7 +2250,7 @@ static GOptionEntry entries_info[] = {
 	{"no-verify", '\0', 0, G_OPTION_ARG_NONE, &verification_disabled, "disable bundle verification", NULL},
 	{"no-check-time", '\0', 0, G_OPTION_ARG_NONE, &no_check_time, "don't check validity period of certificates against current time", NULL},
 	{"key", '\0', G_OPTION_FLAG_NOALIAS, G_OPTION_ARG_FILENAME, &keypath, "decryption key file or PKCS#11 URL", "PEMFILE|PKCS11-URL"},
-	{"output-format", '\0', 0, G_OPTION_ARG_STRING, &output_format, "output format", "FORMAT"},
+	{"output-format", '\0', 0, G_OPTION_ARG_STRING, &output_format, "output format (readable, shell, json, json-pretty, json-2)", "FORMAT"},
 	{"dump-cert", '\0', 0, G_OPTION_ARG_NONE, &info_dumpcert, "dump certificate", NULL},
 	{"dump-recipients", '\0', 0, G_OPTION_ARG_NONE, &info_dumprecipients, "dump recipients", NULL},
 	{0}
@@ -2171,7 +2258,7 @@ static GOptionEntry entries_info[] = {
 
 static GOptionEntry entries_status[] = {
 	{"detailed", '\0', 0, G_OPTION_ARG_NONE, &status_detailed, "show more status details", NULL},
-	{"output-format", '\0', 0, G_OPTION_ARG_STRING, &output_format, "output format", "FORMAT"},
+	{"output-format", '\0', 0, G_OPTION_ARG_STRING, &output_format, "output format (readable, shell, json, json-pretty)", "FORMAT"},
 #if ENABLE_SERVICE == 0
 	{"override-boot-slot", '\0', 0, G_OPTION_ARG_STRING, &bootslot, "override auto-detection of booted slot", "BOOTNAME"},
 #endif
@@ -2265,7 +2352,7 @@ static void create_option_groups(void)
 static void cmdline_handler(int argc, char **argv)
 {
 	gboolean help = FALSE, debug = FALSE, version = FALSE;
-	gchar *confpath = NULL, *keyring = NULL, *mount = NULL;
+	g_autofree gchar *confpath = NULL, *keyring = NULL, *mount = NULL;
 	char *cmdarg = NULL;
 	g_autoptr(GOptionContext) context = NULL;
 	GOptionEntry entries[] = {
@@ -2545,6 +2632,9 @@ int main(int argc, char **argv)
 	fatal_mask |= G_LOG_LEVEL_CRITICAL;
 	g_log_set_always_fatal(fatal_mask);
 
+	/* set up structured logging */
+	g_log_set_writer_func(r_event_log_writer, NULL, NULL);
+
 	/* disable remote VFS */
 	g_assert(g_setenv("GIO_USE_VFS", "local", TRUE));
 
