@@ -214,8 +214,7 @@ static void bundle_test_create_extract(BundleFixture *fixture,
 	g_clear_pointer(&filepath, g_free);
 }
 
-static void bundle_test_create_mount_extract(BundleFixture *fixture,
-		gconstpointer user_data)
+static void bundle_test_create_mount(BundleFixture *fixture, gconstpointer user_data)
 {
 	g_autoptr(RaucBundle) bundle = NULL;
 	g_autoptr(GError) ierror = NULL;
@@ -288,6 +287,7 @@ static void bundle_test_create_check_mount_with_pre_check_corrupt(BundleFixture
 	res = mount_bundle(bundle, &ierror);
 	g_assert_error(ierror, G_FILE_ERROR, G_FILE_ERROR_IO);
 	g_assert_false(res);
+	g_assert_nonnull(strstr(ierror->message, "failed between 1048576 and 1114112 bytes with error: Input/output error"));
 }
 
 static void bundle_test_extract_signature(BundleFixture *fixture,
@@ -762,6 +762,8 @@ int main(int argc, char *argv[])
 	BundleData *bundle_data;
 	setlocale(LC_ALL, "C");
 
+	g_assert(g_setenv("GIO_USE_VFS", "local", TRUE));
+
 	replace_strdup(&r_context_conf()->configpath, "test/test.conf");
 	r_context();
 
@@ -791,9 +793,9 @@ int main(int argc, char *argv[])
 				bundle_fixture_set_up_bundle, bundle_test_create_extract,
 				bundle_fixture_tear_down);
 
-		g_test_add(dup_test_printf(ptrs, "/bundle/create_mount_extract/%s", format_name),
+		g_test_add(dup_test_printf(ptrs, "/bundle/create_mount/%s", format_name),
 				BundleFixture, bundle_data,
-				bundle_fixture_set_up_bundle, bundle_test_create_mount_extract,
+				bundle_fixture_set_up_bundle, bundle_test_create_mount,
 				bundle_fixture_tear_down);
 
 		g_test_add(dup_test_printf(ptrs, "/bundle/extract_signature/%s", format_name),
@@ -841,25 +843,27 @@ int main(int argc, char *argv[])
 				bundle_fixture_set_up_bundle_codesign, bundle_test_purpose_codesign,
 				bundle_fixture_tear_down);
 
-		g_test_add(dup_test_printf(ptrs, "/bundle/create_mount_extract_with_pre_check/%s", format_name),
-				BundleFixture, bundle_data,
-				bundle_fixture_set_up_bundle, bundle_test_create_mount_extract_with_pre_check,
-				bundle_fixture_tear_down);
+		if (format != R_MANIFEST_FORMAT_PLAIN) {
+			g_test_add(dup_test_printf(ptrs, "/bundle/create_mount_extract_with_pre_check/%s", format_name),
+					BundleFixture, bundle_data,
+					bundle_fixture_set_up_bundle, bundle_test_create_mount_extract_with_pre_check,
+					bundle_fixture_tear_down);
 
-		g_test_add(dup_test_printf(ptrs, "/bundle/create_mount_with_pre_check_corrupt/%s", format_name),
-				BundleFixture, bundle_data,
-				bundle_fixture_set_up_bundle_corrupt, bundle_test_create_check_mount_with_pre_check_corrupt,
-				bundle_fixture_tear_down);
+			g_test_add(dup_test_printf(ptrs, "/bundle/create_mount_with_pre_check_corrupt/%s", format_name),
+					BundleFixture, bundle_data,
+					bundle_fixture_set_up_bundle_corrupt, bundle_test_create_check_mount_with_pre_check_corrupt,
+					bundle_fixture_tear_down);
+		}
 	}
 
 	/* test casync manifest contents */
 	g_test_add("/bundle/check_casync/old",
-			BundleFixture, bundle_data,
+			BundleFixture, NULL,
 			bundle_fixture_set_up, bundle_test_check_casync_old,
 			bundle_fixture_tear_down);
 
 	g_test_add("/bundle/check_casync/new",
-			BundleFixture, bundle_data,
+			BundleFixture, NULL,
 			bundle_fixture_set_up, bundle_test_check_casync_new,
 			bundle_fixture_tear_down);
 
