@@ -14,7 +14,7 @@
 #include "../rauc-installer-generated.h"
 
 #include "common.h"
-#include "install-fixtures.h"
+#include "install_fixtures.h"
 
 typedef struct {
 	GTestDBus *dbus;
@@ -24,7 +24,7 @@ typedef struct {
 GMainLoop *testloop = NULL;
 RInstaller *installer = NULL;
 
-static void service_install_fixture_set_up(ServiceFixture *fixture, gconstpointer user_data)
+static void service_fixture_set_up(ServiceFixture *fixture, gconstpointer user_data)
 {
 	g_autofree gchar *contents = NULL;
 	g_autofree gchar *filename = NULL;
@@ -32,11 +32,6 @@ static void service_install_fixture_set_up(ServiceFixture *fixture, gconstpointe
 	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
 
 	fixture_helper_set_up_system(fixture->tmpdir, NULL);
-	fixture_helper_set_up_bundle(fixture->tmpdir, NULL,
-			&(ManifestTestOptions) {
-		.custom_handler = FALSE,
-		.hooks = FALSE,
-	});
 
 	/* Write a D-Bus service file with current tmpdir */
 	contents = g_strdup_printf("\
@@ -50,22 +45,17 @@ Exec="TEST_SERVICES "/rauc -c %s/system.conf --mount=%s/mount --override-boot-sl
 	g_test_dbus_add_service_dir(fixture->dbus, fixture->tmpdir);
 	g_test_dbus_up(fixture->dbus);
 }
-static void service_info_fixture_set_up(ServiceFixture *fixture, gconstpointer user_data)
+
+static void service_install_fixture_set_up(ServiceFixture *fixture, gconstpointer user_data)
 {
-	g_autofree gchar *filename = NULL;
+	service_fixture_set_up(fixture, user_data);
 
-	fixture->tmpdir = g_dir_make_tmp("rauc-XXXXXX", NULL);
-
-	/* Write a D-Bus service file with current tmpdir */
-	filename = write_tmp_file(fixture->tmpdir, "de.pengutronix.rauc.service", "\
-[D-BUS Service]\n\
-Name=de.pengutronix.rauc\n\
-Exec="TEST_SERVICES "/rauc -c test/test.conf --override-boot-slot=system0 service\n", NULL);
-	(void)filename;
-
-	fixture->dbus = g_test_dbus_new(G_TEST_DBUS_NONE);
-	g_test_dbus_add_service_dir(fixture->dbus, fixture->tmpdir);
-	g_test_dbus_up(fixture->dbus);
+	fixture_helper_set_up_bundle(fixture->tmpdir, NULL,
+			&(ManifestTestOptions) {
+		.custom_handler = FALSE,
+		.hooks = FALSE,
+		.slots = TRUE,
+	});
 }
 
 static void service_fixture_tear_down(ServiceFixture *fixture, gconstpointer user_data)
@@ -90,22 +80,24 @@ static void on_installer_changed(GDBusProxy *proxy, GVariant *changed,
 		gpointer data)
 {
 	GQueue *args = data;
-	gchar *msg;
-	GVariant *var;
+	g_autofree gchar *msg = NULL;
 	gint32 percentage, depth;
-	const gchar *message = NULL;
+	g_autofree gchar *message = NULL;
 
 	if (g_variant_lookup(changed, "Operation", "s", &msg)) {
 		g_message("Operation: %s", msg);
 	}
 	if (g_variant_lookup(changed, "Progress", "(isi)", &percentage, &message, &depth)) {
+		g_autoptr(GVariant) var = g_queue_pop_head(args);
 		gint32 cmp_percentage, cmp_depth;
-		const gchar *cmp_message;
+		g_autofree gchar *cmp_message = NULL;
 
-		var = g_queue_pop_head(args);
-		cmp_percentage = g_variant_get_int32(g_variant_get_child_value(var, 0));
-		cmp_message = g_variant_get_string(g_variant_get_child_value(var, 1), NULL);
-		cmp_depth = g_variant_get_int32(g_variant_get_child_value(var, 2));
+		g_assert_nonnull(var);
+
+		g_assert_true(g_variant_is_floating(var));
+		g_variant_get_child(var, 0, "i", &cmp_percentage);
+		g_variant_get_child(var, 1, "s", &cmp_message);
+		g_variant_get_child(var, 2, "i", &cmp_depth);
 		g_message("Progress changed: %03d, %s, %d", percentage, message, depth);
 		g_assert_cmpint(percentage, ==, cmp_percentage);
 		g_assert_cmpstr(message, ==, cmp_message);
@@ -158,7 +150,7 @@ static void assert_progress(GVariant *progress, gint32 percentage, const gchar *
 
 static void service_test_install(ServiceFixture *fixture, gconstpointer user_data, gboolean deprecated)
 {
-	GQueue *args = g_queue_new();
+	g_autoptr(GQueue) args = g_queue_new();
 	const gchar *operation = NULL, *last_error = NULL;
 	GVariant *progress = NULL;
 	const gchar *compatible = NULL;
@@ -169,7 +161,7 @@ static void service_test_install(ServiceFixture *fixture, gconstpointer user_dat
 	gboolean ret = FALSE;
 
 	if (!ENABLE_SERVICE) {
-		g_test_skip("Test requires RAUC being configured with \"--enable-service\".");
+		g_test_skip("Test requires RAUC being configured with \"-Dservice=true\".");
 		return;
 	}
 
@@ -261,6 +253,8 @@ static void service_test_install(ServiceFixture *fixture, gconstpointer user_dat
 	g_main_loop_run(testloop);
 
 	g_clear_object(&installer);
+
+	g_assert_cmpuint(args->length, ==, 0);
 }
 
 static void service_test_install_bundle(ServiceFixture *fixture, gconstpointer user_data)
@@ -275,7 +269,7 @@ static void service_test_install_api(ServiceFixture *fixture, gconstpointer user
 	g_auto(GVariantDict) dict = G_VARIANT_DICT_INIT(NULL);
 
 	if (!ENABLE_SERVICE) {
-		g_test_skip("Test requires RAUC being configured with \"--enable-service\".");
+		g_test_skip("Test requires RAUC being configured with \"-Dservice=true\".");
 		return;
 	}
 
@@ -299,7 +293,7 @@ static void service_test_install_api(ServiceFixture *fixture, gconstpointer user
 
 	ret = r_installer_call_install_bundle_sync(
 			installer,
-			g_strdup("dummy path"),
+			"dummy path",
 			g_variant_dict_end(&dict), /* floating, no unref needed */
 			NULL,
 			&error);
@@ -314,7 +308,7 @@ static void service_test_install_api(ServiceFixture *fixture, gconstpointer user
 
 	ret = r_installer_call_install_bundle_sync(
 			installer,
-			g_strdup("dummy path"),
+			"dummy path",
 			g_variant_dict_end(&dict), /* floating, no unref needed */
 			NULL,
 			&error);
@@ -329,7 +323,7 @@ static void service_test_install_api(ServiceFixture *fixture, gconstpointer user
 
 	ret = r_installer_call_install_bundle_sync(
 			installer,
-			g_strdup("dummy path"),
+			"dummy path",
 			g_variant_dict_end(&dict), /* floating, no unref needed */
 			NULL,
 			&error);
@@ -355,7 +349,7 @@ static void service_test_info(ServiceFixture *fixture, gconstpointer user_data,
 	g_autofree gchar *bundlepath = NULL;
 
 	if (!ENABLE_SERVICE) {
-		g_test_skip("Test requires RAUC being configured with \"--enable-service\".");
+		g_test_skip("Test requires RAUC being configured with \"-Dservice=true\".");
 		return;
 	}
 
@@ -422,7 +416,7 @@ static void service_test_slot_status(ServiceFixture *fixture, gconstpointer user
 	GVariant *slot_status_array = NULL;
 
 	if (!ENABLE_SERVICE) {
-		g_test_skip("Test requires RAUC being configured with \"--enable-service\".");
+		g_test_skip("Test requires RAUC being configured with \"-Dservice=true\".");
 		return;
 	}
 
@@ -456,6 +450,8 @@ int main(int argc, char *argv[])
 {
 	setlocale(LC_ALL, "C");
 
+	g_assert(g_setenv("GIO_USE_VFS", "local", TRUE));
+
 	g_test_init(&argc, &argv, NULL);
 
 	g_test_add("/service/install-bundle", ServiceFixture, NULL,
@@ -471,15 +467,15 @@ int main(int argc, char *argv[])
 			service_fixture_tear_down);
 
 	g_test_add("/service/info-bundle", ServiceFixture, NULL,
-			service_info_fixture_set_up, service_test_info_bundle,
+			service_fixture_set_up, service_test_info_bundle,
 			service_fixture_tear_down);
 
 	g_test_add("/service/info-deprecated", ServiceFixture, NULL,
-			service_info_fixture_set_up, service_test_info_deprecated,
+			service_fixture_set_up, service_test_info_deprecated,
 			service_fixture_tear_down);
 
 	g_test_add("/service/slot-status", ServiceFixture, NULL,
-			service_info_fixture_set_up, service_test_slot_status,
+			service_fixture_set_up, service_test_slot_status,
 			service_fixture_tear_down);
 
 	return g_test_run();
